package be.iminds.ilabt.jfed.ui.javafx.automated_testing_gui;

import be.iminds.ilabt.jfed.fedmon.util.TiParamExpander;
import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClient;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.TestDefinition;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.TestInstance;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import io.dropwizard.jackson.Jackson;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.VBox;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LoadConfigFromUrlController {
    private static final Logger LOG = LoggerFactory.getLogger(LoadConfigFromUrlDialog.class);
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    private FedmonWebApiClient fedmonWebApiClient;

    @FXML private VBox root;
    @FXML public TextField urlField;

    @FXML public Label errorInfoText;

    @FXML public Label tiIdText;
    @FXML public Label tdIdText;
    @FXML public Label testClassText;
    @FXML public Label serverIdText;
    @FXML public Label testGroupText;
    @FXML public Label configText;

    private LoadConfigFromUrlDialog.LoadConfigFromUrlDialogResult currentConfig;

    @Inject
    public LoadConfigFromUrlController(FedmonWebApiClient fedmonWebApiClient) {
        this.fedmonWebApiClient = fedmonWebApiClient;
    }

    @Nullable
    private Integer parseInstanceID(@Nonnull URL url) {
        String path = url.getPath();

        Pattern tiPattern = Pattern.compile("^/?testinstance/([0-9]+)$");
        Matcher tiMatcher = tiPattern.matcher(path);
        if (tiMatcher.matches()) {
            return Integer.parseInt(tiMatcher.group(1));
        }

        Pattern histPattern = Pattern.compile("^/?history/([0-9]+)$");
        Matcher histMatcher = histPattern.matcher(path);
        if (histMatcher.matches()) {
            return Integer.parseInt(histMatcher.group(1));
        }

        return null;
    }

    public LoadConfigFromUrlDialog.LoadConfigFromUrlDialogResult getConfigFromTestInstance(
            @Nonnull TestInstance testInstance,
            @Nonnull TestDefinition testDefinition) {
        Integer serverId = testInstance.getServerIdParameter();
        String testClassName = null;
        String testGroup = null;
        String config = null;

        LOG.debug("getConfigFromTestInstance serverId="+serverId);
        boolean unsupportedTest;

        switch (testDefinition.getType()) {
            //WARNING, this is based on testRunnerFactory

            case "ping":
            case "aggregatetestbedstatus":
            case "resourcestotests":
            case "replayandsummarize":
            case "parallelstitch":
            case "listresources":
            case "selftest":
                unsupportedTest = true;
                break;
            case "automatedtestertestcall":
                unsupportedTest = false;
                testClassName = testInstance.getStringParameterOrDefault("test_class", testDefinition);
                testGroup = testInstance.getStringParameterOrDefault("test_group", testDefinition);
                String contextFileContent = testInstance.getStringParameterOrDefault("context-file", testDefinition);
                break;
            case "automatedtester2testcall": //fall-through
            case "automatedtesterespectestcall":
                unsupportedTest = false;
                testClassName = testInstance.getStringParameterOrDefault("test_class", testDefinition);
                testGroup = testInstance.getStringParameterOrDefault("test_group", testDefinition);
                Object at2config = testInstance.getParameterOrDefault("automated_tester_config", testDefinition);
                if (at2config != null) {
                    if (!(at2config instanceof Map)) {
                        throw new RuntimeException("Non Map configuration specified in automated_tester_config (must be json object)");
                    }
                    Map<?, ?> at2configMap = (Map<?, ?>) at2config;
                    at2configMap = TiParamExpander.expandTiParameters(at2configMap, testInstance);
                    try {
                        config = MAPPER.enable(SerializationFeature.INDENT_OUTPUT).writeValueAsString(at2configMap);
                    } catch (JsonProcessingException e) {
                        LOG.error("Failed to convert config to JSON", e);
                        config = null;
                    }
                } else {
                    config = null;
                }

                break;
            default:
                LOG.warn("Unmapped TestDefinition: "+testInstance.getTestDefinitionId());
                unsupportedTest = true;
                break;
        }

        LOG.debug("Looking up testDefinition "+testInstance.getTestDefinitionId()+"" +
                " of type "+testDefinition.getType()+" unsupportedTest="+unsupportedTest+
                " testClassName="+testClassName+
                " testGroup="+testGroup+
                " config="+config);

        if (unsupportedTest) {
            LOG.info("The TestDefinition does not match a test that the automated tester GUI can run: "+testInstance.getTestDefinitionId());
            return null;
        }

        if (testClassName != null/* && config != null && serverId != null*/) {
            return new LoadConfigFromUrlDialog.LoadConfigFromUrlDialogResult(
                    testClassName,
                    testGroup,
                    config,
                    serverId
            );
        } else {
            return null;
        }
    }

    public void fetch() {
        boolean error = false;
        try {
            errorInfoText.setVisible(false);

            URL url = new URL(urlField.getText());

            /*
             * Example URLs:
             *   Possible:
             *     - https://flsmonitor-api.fed4fire.eu/testinstance/17
             *     - https://flsmonitor.fed4fire.eu/history/17  (last nr is test instance ID)
             *     - https://fedmon.fed4fire.eu/history/669
             *
             *   Possible but not implemented yet:
             *     - https://flsmonitor-api.fed4fire.eu/result?embed=true&last=&testinstanceid=17 (possible to extract from search params)
             *     - https://flsmonitor.fed4fire.eu/results?testinstanceid=17 (possible to extract from search params)
             *     - https://fedmon.fed4fire.eu/result/113631095 (Result contains link to test instance, so not hard)
             *     - https://flsmonitor-api.fed4fire.eu/result/113542183
             *
             *   Maybe possible:
             *     - https://flsmonitor-api.fed4fire.eu/log/213184807/  (would require getting instance from log, wqhich can be txt, html, xml...)
             *              *
             *   Possible but not implemented yet, and only TestDefinition, so can partially fill in, but not all:
             *     - https://flsmonitor-api.fed4fire.eu/testdefinition/stitch
             *     - https://flsmonitor.fed4fire.eu/results?testdefinitionname=stitch
             * */

            Integer testInstanceId = parseInstanceID(url);

            if (testInstanceId != null) {
                tiIdText.setText(testInstanceId+"");
                TestInstance testInstance = fedmonWebApiClient.get(TestInstance.class, testInstanceId).get();
                TestDefinition testDefinition = fedmonWebApiClient.get(TestDefinition.class, testInstance.getTestDefinitionId()).get();
                tiIdText.setText(testInstanceId+" ("+testInstance.getName()+")");
                tdIdText.setText(testDefinition.getId());
                currentConfig = getConfigFromTestInstance(testInstance, testDefinition);
            } else {
                tiIdText.setText("*none*");
                tdIdText.setText("*none*");
                currentConfig = null;
            }


            if (currentConfig != null && currentConfig.getTestClassName() != null) {
                testClassText.setText(currentConfig.getTestClassName());
            } else {
                testClassText.setText("");
            }

            if (currentConfig != null && currentConfig.getTestGroup() != null) {
                testGroupText.setText(currentConfig.getTestGroup());
            } else {
                testGroupText.setText("");
            }

            if (currentConfig != null && currentConfig.getConfig() != null) {
                configText.setText(currentConfig.getConfig());
            } else {
                configText.setText("");
            }

            if (currentConfig != null && currentConfig.getServerId() != null) {
                serverIdText.setText(currentConfig.getServerId()+"");
            } else {
                serverIdText.setText("");
            }
        } catch (MalformedURLException e) {
            LOG.debug("Malformed URL in LoadConfigFromUrlController: \""+urlField.getText()+"\": "+e.getMessage());
            errorInfoText.setText("Malformed URL: "+e.getMessage());
            error = true;
        } catch (FedmonWebApiClient.FedmonWebApiClientException e) {
            LOG.debug("Failure when contacting web API.", e);
            errorInfoText.setText("Failure when contacting web API: "+e.getMessage());
            error = true;
        }

        if (error) {
            currentConfig = null;
            errorInfoText.setVisible(true);
            tiIdText.setText("*error*");
            tdIdText.setText("*error*");
            testClassText.setText("*error*");
            testGroupText.setText("*error*");
            configText.setText("*error*");
            serverIdText.setText("*error*");
        }
    }

    @Nullable
    public LoadConfigFromUrlDialog.LoadConfigFromUrlDialogResult getConfig() {
        return currentConfig;
    }

    @FXML
    public void initialize() {
        //TODO?
    }
}
