package be.iminds.ilabt.jfed.lowlevel.user;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.util.common.GeniUrn;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * GeniUser represents a logged-in user
 */
public interface GeniUser {
    /**
     * @return The Private key of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nonnull
    PrivateKey getPrivateKey();

    /**
     * @return The public key of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nonnull
    PublicKey getPublicKey();

    /**
     * @return The X509 certificate chain of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nonnull
    List<X509Certificate> getClientCertificateChain();

    /**
     * @return The authority server of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     *
     * The only case this can be null is when the authority is not known (yet) by jFed.
     * This is only possible when using a new authority to the scanner or probe.
     */
    @Nullable
    Server getUserAuthorityServer();

    @Nullable
    Integer getUserAuthorityServerId();

    /**
     * @return The URN of the user setting up the SSL connection, in string format.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nonnull
    String getUserUrnString();

    /**
     * @return The URN of the user setting up the SSL connection. In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nonnull
    GeniUrn getUserUrn();


    /**
     * @return The file containing the private key of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nullable
    File getPrivateKeyFile();

    /**
     * @return The file containing the certificate(s) of the user setting up the SSL connection.
     * <p>
     * In speaksfor case, this is the speaker, not to spoken for user.
     */
    @Nullable
    File getCertificateFile();


    /*
    * example getPublicKey implementation:

    @Override
    public PublicKey getPublicKey() {
        List<X509Certificate> certificateChain = getClientCertificateChain();
        if (certificateChain == null || certificateChain.isEmpty())
            return null;
        return certificateChain.get(0).getPublicKey();
    }
    * */

    @Nullable default String getCertEmail() {
        List<X509Certificate> certs = getClientCertificateChain();
        if (certs == null || certs.size() == 0) {
            return null;
        }
        X509Certificate cert = getClientCertificateChain().get(0);
        try {
            for (List<?> list : cert.getSubjectAlternativeNames()) {
                if (Objects.equals(list.get(0), 1)) {  //if 1, then this is a rfc822Name-field
                    return list.get(1).toString();
                }
            }
            return null;
        } catch (CertificateParsingException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Nullable default Date getCertExpire() {
        List<X509Certificate> certs = getClientCertificateChain();
        if (certs == null || certs.size() == 0) {
            return null;
        }
        Date earliest = null;
        for (X509Certificate cert : certs) {
            Date expire = cert.getNotAfter();
            if (earliest == null || earliest.after(expire))
                earliest = expire;
        }
        return earliest;
    }
}
